@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: ike-scan.c,v 1.149 2005/01/13 11:45:30 rsh Exp $
+ * $Id: ike-scan.c,v 1.154 2005/02/09 16:22:59 rsh Exp $
  *
  * ike-scan -- The IKE Scanner
  *
@@ -47,16 +47,14 @@
 
 #include "ike-scan.h"
 
-static const char rcsid[] = "$Id: ike-scan.c,v 1.149 2005/01/13 11:45:30 rsh Exp $";   /* RCS ID for ident(1) */
+static const char rcsid[] = "$Id: ike-scan.c,v 1.154 2005/02/09 16:22:59 rsh Exp $";   /* RCS ID for ident(1) */
 
 /* Global variables */
 struct host_entry *helist = NULL;	/* Dynamic array of host entries */
 struct host_entry **helistptr;		/* Array of pointers to host entries */
 struct host_entry **cursor;		/* Pointer to current list entry */
 struct pattern_list *patlist = NULL;	/* Backoff pattern list */
-#ifdef HAVE_REGEX_H
 struct vid_pattern_list *vidlist = NULL;	/* Vendor ID pattern list */
-#endif
 static int verbose=0;			/* Verbose level */
 unsigned experimental_value=0;		/* Experimental value */
 int tcp_flag=0;				/* TCP flag */
@@ -429,9 +427,7 @@ main(int argc, char *argv[]) {
 /*
  *	Load known Vendor ID patterns from the Vendor ID file.
  */
-#ifdef HAVE_REGEX_H
    load_vid_patterns(vidfile);
-#endif
 /*
  *	Check that we have at least one entry in the list.
  */
@@ -500,8 +496,12 @@ main(int argc, char *argv[]) {
       if ((setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on))) < 0)
          err_sys("ERROR: setsockopt() failed");
    } else {
+      const int on = 1;	/* for setsockopt() */
+
       if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
          err_sys("ERROR: socket");
+      if ((setsockopt(sockfd, SOL_SOCKET, SO_BROADCAST, &on, sizeof(on))) != 0)
+         err_sys("setsockopt");
    }
 
    memset(&sa_local, '\0', sizeof(sa_local));
@@ -582,12 +582,9 @@ main(int argc, char *argv[]) {
       dump_list(num_hosts);
       if (showbackoff_flag)
          dump_backoff(pattern_fuzz);
-#ifdef HAVE_REGEX_H
       dump_vid();
-#else
       warn_msg("This ike-scan binary was not compiled with Posix regular expression support.");
       warn_msg("Vendor ID Fingerprinting will not be possible.");
-#endif
    }
 /*
  *	Main loop: send packets to all hosts in order until a response
@@ -811,16 +808,48 @@ add_host_pattern(const char *pattern, unsigned timeout, unsigned *num_hosts) {
    unsigned long hoststart;
    unsigned long hostend;
    unsigned i;
+   static int first_call=1;
+   static regex_t iprange_pat;
+   static regex_t ipslash_pat;
+   static const char *iprange_pat_str = 
+      "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+-[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+";
+   static const char *ipslash_pat_str = 
+      "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/[0-9]+";
+/*
+ *	Compile regex patterns if this is the first time we've been called.
+ */
+   if (first_call) {
+      int result;
+
+      first_call = 0;
+      if ((result=regcomp(&iprange_pat, iprange_pat_str,
+                          REG_EXTENDED|REG_NOSUB))) {
+         char errbuf[MAXLINE];
+         size_t errlen;
+         errlen=regerror(result, &iprange_pat, errbuf, MAXLINE);
+         err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
+                 iprange_pat_str, errbuf);
+      }
+      if ((result=regcomp(&ipslash_pat, ipslash_pat_str,
+                          REG_EXTENDED|REG_NOSUB))) {
+         char errbuf[MAXLINE];
+         size_t errlen;
+         errlen=regerror(result, &ipslash_pat, errbuf, MAXLINE);
+         err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
+                 ipslash_pat_str, errbuf);
+      }
+   }
 /*
  *	Make a copy of pattern because we don't want to modify our argument.
  */
    patcopy=Malloc(strlen(pattern)+1);
    strcpy(patcopy, pattern);
 
-   if        ((cp=strchr(patcopy, '/'))) {	/* IPnet/bits */
+   if (!(regexec(&ipslash_pat, patcopy, 0, NULL, 0))) { /* IPnet/bits */
 /*
  *	Get IPnet and bits as integers.  Perform basic error checking.
  */
+      cp=strchr(patcopy, '/');
       *(cp++)='\0';	/* patcopy points to IPnet, cp points to bits */
       if (!(inet_aton(patcopy, &in_val)))
          err_msg("ERROR: %s is not a valid IP address", patcopy);
@@ -865,10 +894,11 @@ add_host_pattern(const char *pattern, unsigned timeout, unsigned *num_hosts) {
          sprintf(ipstr, "%d.%d.%d.%d", b1,b2,b3,b4);
          add_host(ipstr, timeout, num_hosts);
       }
-   } else if ((cp=strchr(patcopy, '-'))) {	/* IPstart-IPend */
+   } else if (!(regexec(&iprange_pat, patcopy, 0, NULL, 0))) { /* IPstart-IPend */
 /*
  *	Get IPstart and IPend as integers.
  */
+      cp=strchr(patcopy, '-');
       *(cp++)='\0';	/* patcopy points to IPstart, cp points to IPend */
       if (!(inet_aton(patcopy, &in_val)))
          err_msg("ERROR: %s is not a valid IP address", patcopy);
@@ -891,7 +921,7 @@ add_host_pattern(const char *pattern, unsigned timeout, unsigned *num_hosts) {
          sprintf(ipstr, "%d.%d.%d.%d", b1,b2,b3,b4);
          add_host(ipstr, timeout, num_hosts);
       }
-   } else {				/* Single host */
+   } else {				/* Single host or IP address */
       add_host(patcopy, timeout, num_hosts);
    }
    free(patcopy);
@@ -1109,21 +1139,21 @@ display_packet(int n, unsigned char *packet_in, struct host_entry *he,
    char *msg;			/* Message to display */
    char *msg2;
    unsigned char *pkt_ptr;
-   static const char *payload_name[] = { /* Payload types from RFC 2408 3.1 */
-      NULL,                             /* 0 */
-      "SecurityAssociation",		/* 1 */
-      "Proposal",                       /* 2 */
-      "Transform",                      /* 3 */
-      "KeyExchange",			/* 4 */
-      "Identification",                 /* 5 */
-      "Certificate",                    /* 6 */
-      "CertificateRequest",		/* 7 */
-      "Hash",                           /* 8 */
-      "Signature",                      /* 9 */
-      "Nonce",                          /* 10 */
-      "Notification",                   /* 11 */
-      "Delete",                         /* 12 */
-      "VendorID"			/* 13 */
+   static const id_name_map payload_map[] = { /* Payload types RFC 2408 3.1 */
+      {1, "SecurityAssociation"},
+      {2, "Proposal"},
+      {3, "Transform"},
+      {4, "KeyExchange"},
+      {5, "Identification"},
+      {6, "Certificate"},
+      {7, "CertificateRequest"},
+      {8, "Hash"},
+      {9, "Signature"},
+      {10, "Nonce"},
+      {11, "Notification"},
+      {12, "Delete"},
+      {13, "VendorID"},
+      {-1, NULL}
    };
 
 /*
@@ -1167,7 +1197,7 @@ display_packet(int n, unsigned char *packet_in, struct host_entry *he,
          break;
       default:
          cp=make_message("Unknown IKE payload returned: %s",
-                         STR_OR_ID(next, payload_name));
+                         id_to_name(next, payload_map));
          break;
    }
    pkt_ptr = skip_payload(pkt_ptr, &bytes_left, &next);
@@ -1200,11 +1230,11 @@ display_packet(int n, unsigned char *packet_in, struct host_entry *he,
             msg2=msg;
             if (bytes_left >= sizeof(struct isakmp_generic)) {
                 struct isakmp_generic *hdr = (struct isakmp_generic *) pkt_ptr;
-                cp=make_message("%s(%u bytes)", STR_OR_ID(next, payload_name),
+                cp=make_message("%s(%u bytes)", id_to_name(next, payload_map),
                                 ntohs(hdr->isag_length) -
                                 sizeof(struct isakmp_generic));
             } else {
-                cp=make_message("%s)", STR_OR_ID(next, payload_name));
+                cp=make_message("%s)", id_to_name(next, payload_map));
             }
             msg=make_message("%s%s%s", msg2, multiline?"\n\t":" ", cp);
             free(msg2);	/* Free old message */
@@ -1723,7 +1753,6 @@ dump_backoff(unsigned pattern_fuzz) {
  *	when debugging to check that the patterns have been loaded correctly
  *	from the Vendor ID patterns file.
  */
-#ifdef HAVE_REGEX_H
 void
 dump_vid(void) {
    struct vid_pattern_list *pl;
@@ -1739,7 +1768,6 @@ dump_vid(void) {
    } /* End While */
    printf("\nTotal of %d Vendor ID pattern entries.\n\n", i-1);
 }
-#endif
 
 /*
  *	dump_times -- Display packet times for backoff fingerprinting
@@ -2125,7 +2153,6 @@ add_pattern(char *line, unsigned pattern_fuzz) {
  *
  *	None
  */
-#ifdef HAVE_REGEX_H
 void
 load_vid_patterns(const char *vidfile) {
    FILE *fp;
@@ -2172,7 +2199,6 @@ load_vid_patterns(const char *vidfile) {
    }
    free(fn);
 }
-#endif
 
 /*
  *	add_vid_pattern -- add a Vendor ID pattern to the list.
@@ -2185,7 +2211,6 @@ load_vid_patterns(const char *vidfile) {
  *
  *	None.
  */
-#ifdef HAVE_REGEX_H
 void
 add_vid_pattern(char *line) {
    char *cp;
@@ -2266,7 +2291,6 @@ add_vid_pattern(char *line) {
       pe->name = cp;
    }
 }
-#endif
 
 /*
  *	decode_trans -- Decode a custom transform specification
@@ -2338,27 +2362,27 @@ decode_trans(char *str, unsigned *enc, unsigned *keylen, unsigned *hash,
 void
 usage(int status) {
 
-   static const char *auth_methods[] = { /* Auth methods from RFC 2409 App. A */
-      NULL,			/* 0 */
-      "pre-shared key",		/* 1 */
-      "DSS signatures",		/* 2 */
-      "RSA signatures",		/* 3 */
-      "Encryption with RSA",	/* 4 */
-      "Revised encryption with RSA" /* 5 */
+   static const id_name_map auth_map[] = { /* Auth methods RFC 2409 App. A */
+      {1, "pre-shared key"},
+      {2, "DSS signatures"},
+      {3, "RSA signatures"},
+      {4, "Encryption with RSA"},
+      {5, "Revised encryption with RSA"},
+      {-1, NULL}
    };
-   static const char *id_type_name[] ={	/* ID Type names from RFC 2407 4.6.2.1 */
-      NULL,	                /* 0 */
-      "ID_IPV4_ADDR",           /* 1 */
-      "ID_FQDN",                /* 2 */
-      "ID_USER_FQDN",           /* 3 */
-      "ID_IPV4_ADDR_SUBNET",    /* 4 */
-      "ID_IPV6_ADDR",           /* 5 */
-      "ID_IPV6_ADDR_SUBNET",    /* 6 */
-      "ID_IPV4_ADDR_RANGE",     /* 7 */
-      "ID_IPV6_ADDR_RANGE",     /* 8 */
-      "ID_DER_ASN1_DN",         /* 9 */
-      "ID_DER_ASN1_GN",         /* 10 */
-      "ID_KEY_ID"               /* 11 */
+   static const id_name_map id_type_map[] ={ /* ID Type names RFC 2407 4.6.2.1 */
+      {1, "ID_IPV4_ADDR"},
+      {2, "ID_FQDN"},
+      {3, "ID_USER_FQDN"},
+      {4, "ID_IPV4_ADDR_SUBNET"},
+      {5, "ID_IPV6_ADDR"},
+      {6, "ID_IPV6_ADDR_SUBNET"},
+      {7, "ID_IPV4_ADDR_RANGE"},
+      {8, "ID_IPV6_ADDR_RANGE"},
+      {9, "ID_DER_ASN1_DN"},
+      {10, "ID_DER_ASN1_GN"},
+      {11, "ID_KEY_ID"},
+      {-1, NULL}
    };
 
 
@@ -2453,7 +2477,7 @@ usage(int status) {
    fprintf(stderr, "\t\t\twith the --trans options to produce multiple transform\n");
    fprintf(stderr, "\t\t\tpayloads with different lifesizes.  Each --trans option\n");
    fprintf(stderr, "\t\t\twill use the previously specified lifesize value.\n");
-   fprintf(stderr, "\n--auth=<n> or -m <n>\tSet auth. method to <n>, default=%u (%s).\n", DEFAULT_AUTH_METHOD, STR_OR_ID(DEFAULT_AUTH_METHOD, auth_methods));
+   fprintf(stderr, "\n--auth=<n> or -m <n>\tSet auth. method to <n>, default=%u (%s).\n", DEFAULT_AUTH_METHOD, id_to_name(DEFAULT_AUTH_METHOD, auth_map));
    fprintf(stderr, "\t\t\tRFC defined values are 1 to 5.  See RFC 2409 Appendix A.\n");
    fprintf(stderr, "\t\t\tCheckpoint hybrid mode is 64221.\n");
    fprintf(stderr, "\t\t\tGSS (Windows \"Kerberos\") is 65001.\n");
@@ -2521,7 +2545,7 @@ usage(int status) {
    fprintf(stderr, "\t\t\tThis option is only applicable to Aggressive Mode.\n");
    fprintf(stderr, "\t\t\t<id> can be specified as a string, e.g. --id=test or as\n");
    fprintf(stderr, "\t\t\ta hex value with a leading \"0x\", e.g. --id=0xdeadbeef.\n");
-   fprintf(stderr, "\n--idtype=<n> or -y <n>\tUse identification type <n>.  Default %u (%s).\n", DEFAULT_IDTYPE, STR_OR_ID(DEFAULT_IDTYPE, id_type_name));
+   fprintf(stderr, "\n--idtype=<n> or -y <n>\tUse identification type <n>.  Default %u (%s).\n", DEFAULT_IDTYPE, id_to_name(DEFAULT_IDTYPE, id_type_map));
    fprintf(stderr, "\t\t\tThis option is only applicable to Aggressive Mode.\n");
    fprintf(stderr, "\t\t\tSee RFC 2407 4.6.2 for details of Identification types.\n");
    fprintf(stderr, "\n--dhgroup=<n> or -g <n>\tUse Diffie Hellman Group <n>.  Default %u.\n", DEFAULT_DH_GROUP);
