@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: ike-scan.h,v 1.91 2005/01/13 11:07:32 rsh Exp $
+ * $Id: ike-scan.h,v 1.95 2005/01/28 14:04:26 rsh Exp $
  *
  * ike-scan.h -- Header file for IKE Scanner (ike-scan)
  *
@@ -181,9 +181,6 @@ unsigned char *SHA1(const unsigned char *, size_t, unsigned char *);
  */
 #undef ALPHA				/* Smoothing factor */
 
-#define STR_OR_ID(x, tab) \
-        (((x) < sizeof(tab)/sizeof(tab[0]) && tab[(x)]) ? tab[(x)] : numstr(x))
-
 /* Structures */
 struct host_entry {
    struct time_list *recv_times; /* List of receive times */
@@ -215,14 +212,12 @@ struct pattern_list {
    struct pattern_list *next;
 };
 
-#ifdef HAVE_REGEX_H
 struct vid_pattern_list {
    char *name;
    char *pattern;	/* Text regular expression */
    regex_t *regex;	/* Compiled regular expression */
    struct vid_pattern_list *next;
 };
-#endif
 
 struct psk_crack {
    unsigned char *g_xr;		/* Responder DH public value */
@@ -245,6 +240,11 @@ struct psk_crack {
    size_t hash_r_len;
 };
 
+typedef struct {
+   int id;			/* IKE IDs are generally 8 or 16-bits */
+   char *name;
+} id_name_map;
+
 struct ike_udphdr {
   uint16_t     source;
   uint16_t     dest;
@@ -336,6 +336,7 @@ unsigned char *hmac_md5(const unsigned char *, size_t,
                         const unsigned char *, size_t, unsigned char *);
 unsigned char *hmac_sha1(const unsigned char *, size_t,
                          const unsigned char *, size_t, unsigned char *);
+char *id_to_name(int, const id_name_map[]);
 /* The following functions are just to prevent rcsid being optimised away */
 void error_use_rcsid(void);
 void isakmp_use_rcsid(void);
