@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: utils.c,v 1.8 2005/01/01 16:49:53 rsh Exp $
+ * $Id: utils.c,v 1.11 2005/01/21 13:36:06 rsh Exp $
  *
  * Author: Roy Hills
  * Date: 5 April 2004
@@ -35,7 +35,7 @@
 
 #include "ike-scan.h"
 
-static char rcsid[] = "$Id: utils.c,v 1.8 2005/01/01 16:49:53 rsh Exp $";	/* RCS ID for ident(1) */
+static char rcsid[] = "$Id: utils.c,v 1.11 2005/01/21 13:36:06 rsh Exp $";	/* RCS ID for ident(1) */
 
 
 /*
@@ -266,7 +266,6 @@ make_message(const char *fmt, ...) {
  *
  *	Pointer to the string representation of the number.
  *
- *	This is used by the STR_OR_ID macro.
  *	I'm surprised that there is not a standard library function to do this.
  */
 char *
@@ -807,6 +806,45 @@ void sig_alarm(int signo) {
    return;      /* just interrupt the current system call */
 }
 
+/*
+ *	id_to_name -- Return name associated with given id, or id number
+ *
+ *	Inputs:
+ *
+ *	id		The id to find in the map
+ *	id_name_map	Pointer to the id-to-name map
+ *
+ *	Returns:
+ *
+ *	A pointer to the name associated with the id if an association is
+ *	found in the map, otherwise the numeric id.  Returns NULL on error.
+ *
+ *	This function uses a sequential search through the map to find the
+ *	ID and associated name.  This is OK when the map is relatively small,
+ *	but could be time consuming if the map contains a large number of
+ *	entries.
+ */
+char *id_to_name(int id, const id_name_map map[]) {
+   int found = 0;
+   int i = 0;
+
+   if (map == NULL)
+      return NULL;
+
+   while (map[i].id != -1) {
+      if (id == map[i].id) {
+         found = 1;
+         break;
+      }
+      i++;
+   }
+
+   if (found)
+      return map[i].name;
+   else
+      return numstr(id);
+}
+
 void utils_use_rcsid(void) {
    fprintf(stderr, "%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
 }
